
#(MIN-env) C:\Users\Asus\Desktop\UI>python C:\Users\Asus\anaconda3\envs\MIN-env\Library\bin\pyside2-uic PIV.ui 
# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'PIV.ui',
# licensing of 'PIV.ui' applies.
#
# Created: Thu Jul  9 10:41:19 2020
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setEnabled(True)
        MainWindow.resize(956, 566)
        MainWindow.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        MainWindow.setAcceptDrops(False)
        MainWindow.setWindowOpacity(1.0)
        MainWindow.setAutoFillBackground(False)
        MainWindow.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.Main_tabs = QtWidgets.QTabWidget(self.centralwidget)
        self.Main_tabs.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setWeight(50)
        font.setItalic(False)
        font.setUnderline(False)
        font.setStrikeOut(False)
        font.setBold(False)
        self.Main_tabs.setFont(font)
        self.Main_tabs.setAcceptDrops(False)
        self.Main_tabs.setAccessibleDescription("")
        self.Main_tabs.setAutoFillBackground(False)
        self.Main_tabs.setTabPosition(QtWidgets.QTabWidget.North)
        self.Main_tabs.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.Main_tabs.setElideMode(QtCore.Qt.ElideNone)
        self.Main_tabs.setUsesScrollButtons(True)
        self.Main_tabs.setDocumentMode(False)
        self.Main_tabs.setTabsClosable(False)
        self.Main_tabs.setMovable(False)
        self.Main_tabs.setTabBarAutoHide(False)
        self.Main_tabs.setObjectName("Main_tabs")
        self.Validation = QtWidgets.QWidget()
        self.Validation.setEnabled(True)
        self.Validation.setObjectName("Validation")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.Validation)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.splitter = QtWidgets.QSplitter(self.Validation)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.settings_GB = QtWidgets.QGroupBox(self.layoutWidget)
        self.settings_GB.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.settings_GB.sizePolicy().hasHeightForWidth())
        self.settings_GB.setSizePolicy(sizePolicy)
        self.settings_GB.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setWeight(50)
        font.setItalic(False)
        font.setBold(False)
        self.settings_GB.setFont(font)
        self.settings_GB.setAutoFillBackground(False)
        self.settings_GB.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.settings_GB.setFlat(False)
        self.settings_GB.setCheckable(False)
        self.settings_GB.setChecked(False)
        self.settings_GB.setObjectName("settings_GB")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.settings_GB)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.global_velocity_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.global_velocity_CB.setEnabled(True)
        self.global_velocity_CB.setObjectName("global_velocity_CB")
        self.gridLayout.addWidget(self.global_velocity_CB, 1, 0, 1, 1)
        self.global_uVelocity_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.global_uVelocity_LE.setObjectName("global_uVelocity_LE")
        self.gridLayout.addWidget(self.global_uVelocity_LE, 1, 1, 1, 1)
        self.global_vVelocity_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.global_vVelocity_LE.setObjectName("global_vVelocity_LE")
        self.gridLayout.addWidget(self.global_vVelocity_LE, 1, 2, 1, 1)
        self.local_velocity_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.local_velocity_CB.setEnabled(True)
        self.local_velocity_CB.setObjectName("local_velocity_CB")
        self.gridLayout.addWidget(self.local_velocity_CB, 2, 0, 1, 1)
        self.local_velocity_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.local_velocity_LE.setObjectName("local_velocity_LE")
        self.gridLayout.addWidget(self.local_velocity_LE, 2, 1, 1, 1)
        self.local_kernel_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.local_kernel_LE.setObjectName("local_kernel_LE")
        self.gridLayout.addWidget(self.local_kernel_LE, 2, 2, 1, 1)
        self.s2n_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.s2n_CB.setEnabled(True)
        self.s2n_CB.setObjectName("s2n_CB")
        self.gridLayout.addWidget(self.s2n_CB, 0, 0, 1, 1)
        self.global_std_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.global_std_CB.setEnabled(True)
        self.global_std_CB.setObjectName("global_std_CB")
        self.gridLayout.addWidget(self.global_std_CB, 3, 0, 1, 2)
        self.s2n_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.s2n_LE.setEnabled(True)
        self.s2n_LE.setText("")
        self.s2n_LE.setFrame(True)
        self.s2n_LE.setObjectName("s2n_LE")
        self.gridLayout.addWidget(self.s2n_LE, 0, 2, 1, 1)
        self.global_std_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.global_std_LE.setObjectName("global_std_LE")
        self.gridLayout.addWidget(self.global_std_LE, 3, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.apply_settings_PB = QtWidgets.QPushButton(self.settings_GB)
        self.apply_settings_PB.setObjectName("apply_settings_PB")
        self.horizontalLayout.addWidget(self.apply_settings_PB)
        self.load_settings_PB = QtWidgets.QPushButton(self.settings_GB)
        self.load_settings_PB.setObjectName("load_settings_PB")
        self.horizontalLayout.addWidget(self.load_settings_PB)
        self.save_settings_PB = QtWidgets.QPushButton(self.settings_GB)
        self.save_settings_PB.setEnabled(True)
        self.save_settings_PB.setObjectName("save_settings_PB")
        self.horizontalLayout.addWidget(self.save_settings_PB)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.gridLayout_5.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.settings_GB)
        self.files_TW = QtWidgets.QTreeWidget(self.layoutWidget)
        self.files_TW.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.files_TW.sizePolicy().hasHeightForWidth())
        self.files_TW.setSizePolicy(sizePolicy)
        self.files_TW.setMinimumSize(QtCore.QSize(0, 0))
        self.files_TW.setAutoScroll(True)
        self.files_TW.setIndentation(0)
        self.files_TW.setItemsExpandable(True)
        self.files_TW.setAnimated(True)
        self.files_TW.setAllColumnsShowFocus(False)
        self.files_TW.setWordWrap(False)
        self.files_TW.setHeaderHidden(False)
        self.files_TW.setObjectName("files_TW")
        self.files_TW.header().setSortIndicatorShown(True)
        self.files_TW.header().setStretchLastSection(True)
        self.verticalLayout.addWidget(self.files_TW)
        self.plot_widget = QtWidgets.QWidget(self.splitter)
        self.plot_widget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.plot_widget.sizePolicy().hasHeightForWidth())
        self.plot_widget.setSizePolicy(sizePolicy)
        self.plot_widget.setMinimumSize(QtCore.QSize(450, 0))
        self.plot_widget.setAutoFillBackground(False)
        self.plot_widget.setObjectName("plot_widget")
        self.gridLayout_4.addWidget(self.splitter, 0, 0, 1, 1)
        self.Main_tabs.addTab(self.Validation, "")
        self.Process = QtWidgets.QWidget()
        self.Process.setObjectName("Process")
        self.label_4 = QtWidgets.QLabel(self.Process)
        self.label_4.setGeometry(QtCore.QRect(13, 13, 451, 22))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.Main_tabs.addTab(self.Process, "")
        self.FrequencyAnalysis = QtWidgets.QWidget()
        self.FrequencyAnalysis.setObjectName("FrequencyAnalysis")
        self.label_5 = QtWidgets.QLabel(self.FrequencyAnalysis)
        self.label_5.setGeometry(QtCore.QRect(13, 13, 431, 22))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.Main_tabs.addTab(self.FrequencyAnalysis, "")
        self.ModalAnalysis = QtWidgets.QWidget()
        self.ModalAnalysis.setObjectName("ModalAnalysis")
        self.label_6 = QtWidgets.QLabel(self.ModalAnalysis)
        self.label_6.setGeometry(QtCore.QRect(13, 13, 471, 22))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.Main_tabs.addTab(self.ModalAnalysis, "")
        self.gridLayout_3.addWidget(self.Main_tabs, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 956, 31))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.actionload_files = QtWidgets.QAction(MainWindow)
        self.actionload_files.setObjectName("actionload_files")
        self.actionexit = QtWidgets.QAction(MainWindow)
        self.actionexit.setObjectName("actionexit")
        self.menuFile.addAction(self.actionload_files)
        self.menuFile.addAction(self.actionexit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        self.Main_tabs.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtWidgets.QApplication.translate("MainWindow", "PIV - Build 101", None, -1))
        self.settings_GB.setTitle(QtWidgets.QApplication.translate("MainWindow", "Thresholding Settings", None, -1))
        self.global_velocity_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Global Velocity", None, -1))
        self.global_uVelocity_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Umin, Umax", None, -1))
        self.global_vVelocity_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Vmin, Vmax", None, -1))
        self.local_velocity_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Local Median", None, -1))
        self.local_velocity_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Udiff, Vdiff", None, -1))
        self.local_kernel_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "kernel size", None, -1))
        self.s2n_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Signal 2 Noise", None, -1))
        self.global_std_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Global Std. Deviation", None, -1))
        self.s2n_LE.setWhatsThis(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p><br/></p></body></html>", None, -1))
        self.s2n_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "signal/noise", None, -1))
        self.global_std_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "std multiplier", None, -1))
        self.apply_settings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Apply Settings", None, -1))
        self.load_settings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Load Settings", None, -1))
        self.save_settings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Save Settings", None, -1))
        self.files_TW.setSortingEnabled(True)
        self.files_TW.headerItem().setText(0, QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
        self.files_TW.headerItem().setText(1, QtWidgets.QApplication.translate("MainWindow", "Num. Bad Vector", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.Validation), QtWidgets.QApplication.translate("MainWindow", "Validation", None, -1))
        self.label_4.setText(QtWidgets.QApplication.translate("MainWindow", "This part of the program is still under development.", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.Process), QtWidgets.QApplication.translate("MainWindow", "Process", None, -1))
        self.label_5.setText(QtWidgets.QApplication.translate("MainWindow", "This part of the program is still under development.", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.FrequencyAnalysis), QtWidgets.QApplication.translate("MainWindow", "Freq. Analysis", None, -1))
        self.label_6.setText(QtWidgets.QApplication.translate("MainWindow", "This part of the program is still under development.", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.ModalAnalysis), QtWidgets.QApplication.translate("MainWindow", "Modal Analysis", None, -1))
        self.menuFile.setTitle(QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
        self.actionload_files.setText(QtWidgets.QApplication.translate("MainWindow", "Load Files", None, -1))
        self.actionexit.setText(QtWidgets.QApplication.translate("MainWindow", "Exit", None, -1))

