
# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_piv.ui',
# licensing of 'main_piv.ui' applies.
#
# Created: Tue Nov 10 11:22:38 2020
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.setEnabled(True)
        MainWindow.resize(876, 720)
        MainWindow.setStyleSheet("")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_11.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.Main_tabs = QtWidgets.QTabWidget(self.centralwidget)
        self.Main_tabs.setEnabled(True)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setWeight(50)
        font.setItalic(False)
        font.setUnderline(False)
        font.setStrikeOut(False)
        font.setBold(False)
        self.Main_tabs.setFont(font)
        self.Main_tabs.setAcceptDrops(False)
        self.Main_tabs.setAccessibleDescription("")
        self.Main_tabs.setAutoFillBackground(False)
        self.Main_tabs.setStyleSheet("")
        self.Main_tabs.setTabPosition(QtWidgets.QTabWidget.North)
        self.Main_tabs.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.Main_tabs.setElideMode(QtCore.Qt.ElideNone)
        self.Main_tabs.setUsesScrollButtons(True)
        self.Main_tabs.setDocumentMode(False)
        self.Main_tabs.setTabsClosable(False)
        self.Main_tabs.setMovable(False)
        self.Main_tabs.setTabBarAutoHide(False)
        self.Main_tabs.setObjectName("Main_tabs")
        self.Validation = QtWidgets.QWidget()
        self.Validation.setEnabled(True)
        self.Validation.setObjectName("Validation")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.Validation)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.splitter = QtWidgets.QSplitter(self.Validation)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setSpacing(5)
        self.verticalLayout.setContentsMargins(5, 10, 5, 5)
        self.verticalLayout.setObjectName("verticalLayout")
        self.settings_GB = QtWidgets.QGroupBox(self.layoutWidget)
        self.settings_GB.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.settings_GB.sizePolicy().hasHeightForWidth())
        self.settings_GB.setSizePolicy(sizePolicy)
        self.settings_GB.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setWeight(50)
        font.setItalic(False)
        font.setBold(False)
        self.settings_GB.setFont(font)
        self.settings_GB.setAutoFillBackground(False)
        self.settings_GB.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.settings_GB.setFlat(False)
        self.settings_GB.setCheckable(False)
        self.settings_GB.setChecked(False)
        self.settings_GB.setObjectName("settings_GB")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.settings_GB)
        self.gridLayout_5.setSpacing(9)
        self.gridLayout_5.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(8)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setHorizontalSpacing(9)
        self.gridLayout.setObjectName("gridLayout")
        self.global_velocity_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.global_velocity_CB.setEnabled(True)
        self.global_velocity_CB.setObjectName("global_velocity_CB")
        self.gridLayout.addWidget(self.global_velocity_CB, 1, 0, 1, 1)
        self.global_uVelocity_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.global_uVelocity_LE.setObjectName("global_uVelocity_LE")
        self.gridLayout.addWidget(self.global_uVelocity_LE, 1, 1, 1, 1)
        self.global_vVelocity_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.global_vVelocity_LE.setObjectName("global_vVelocity_LE")
        self.gridLayout.addWidget(self.global_vVelocity_LE, 1, 2, 1, 1)
        self.local_velocity_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.local_velocity_CB.setEnabled(True)
        self.local_velocity_CB.setObjectName("local_velocity_CB")
        self.gridLayout.addWidget(self.local_velocity_CB, 2, 0, 1, 1)
        self.local_velocity_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.local_velocity_LE.setObjectName("local_velocity_LE")
        self.gridLayout.addWidget(self.local_velocity_LE, 2, 1, 1, 1)
        self.local_kernel_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.local_kernel_LE.setObjectName("local_kernel_LE")
        self.gridLayout.addWidget(self.local_kernel_LE, 2, 2, 1, 1)
        self.s2n_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.s2n_CB.setEnabled(True)
        self.s2n_CB.setObjectName("s2n_CB")
        self.gridLayout.addWidget(self.s2n_CB, 0, 0, 1, 1)
        self.global_std_CB = QtWidgets.QCheckBox(self.settings_GB)
        self.global_std_CB.setEnabled(True)
        self.global_std_CB.setObjectName("global_std_CB")
        self.gridLayout.addWidget(self.global_std_CB, 3, 0, 1, 2)
        self.s2n_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.s2n_LE.setEnabled(True)
        self.s2n_LE.setText("")
        self.s2n_LE.setFrame(True)
        self.s2n_LE.setObjectName("s2n_LE")
        self.gridLayout.addWidget(self.s2n_LE, 0, 2, 1, 1)
        self.global_std_LE = QtWidgets.QLineEdit(self.settings_GB)
        self.global_std_LE.setObjectName("global_std_LE")
        self.gridLayout.addWidget(self.global_std_LE, 3, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(12)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.apply_settings_PB = QtWidgets.QPushButton(self.settings_GB)
        self.apply_settings_PB.setObjectName("apply_settings_PB")
        self.horizontalLayout.addWidget(self.apply_settings_PB)
        self.load_settings_PB = QtWidgets.QPushButton(self.settings_GB)
        self.load_settings_PB.setObjectName("load_settings_PB")
        self.horizontalLayout.addWidget(self.load_settings_PB)
        self.save_settings_PB = QtWidgets.QPushButton(self.settings_GB)
        self.save_settings_PB.setEnabled(True)
        self.save_settings_PB.setObjectName("save_settings_PB")
        self.horizontalLayout.addWidget(self.save_settings_PB)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.gridLayout_5.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.verticalLayout.addWidget(self.settings_GB)
        self.files_TW = QtWidgets.QTreeWidget(self.layoutWidget)
        self.files_TW.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.files_TW.sizePolicy().hasHeightForWidth())
        self.files_TW.setSizePolicy(sizePolicy)
        self.files_TW.setMinimumSize(QtCore.QSize(0, 0))
        self.files_TW.setAutoScroll(True)
        self.files_TW.setIndentation(0)
        self.files_TW.setItemsExpandable(True)
        self.files_TW.setAnimated(True)
        self.files_TW.setAllColumnsShowFocus(False)
        self.files_TW.setWordWrap(False)
        self.files_TW.setHeaderHidden(False)
        self.files_TW.setObjectName("files_TW")
        self.files_TW.header().setSortIndicatorShown(True)
        self.verticalLayout.addWidget(self.files_TW)
        self.plot_widget = QtWidgets.QWidget(self.splitter)
        self.plot_widget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.plot_widget.sizePolicy().hasHeightForWidth())
        self.plot_widget.setSizePolicy(sizePolicy)
        self.plot_widget.setMinimumSize(QtCore.QSize(450, 0))
        self.plot_widget.setAutoFillBackground(False)
        self.plot_widget.setObjectName("plot_widget")
        self.plot_settings_LE = QtWidgets.QLineEdit(self.plot_widget)
        self.plot_settings_LE.setGeometry(QtCore.QRect(160, 30, 70, 30))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.plot_settings_LE.sizePolicy().hasHeightForWidth())
        self.plot_settings_LE.setSizePolicy(sizePolicy)
        self.plot_settings_LE.setMinimumSize(QtCore.QSize(70, 0))
        self.plot_settings_LE.setMaximumSize(QtCore.QSize(70, 16777215))
        self.plot_settings_LE.setObjectName("plot_settings_LE")
        self.BV_settings_CB = QtWidgets.QComboBox(self.plot_widget)
        self.BV_settings_CB.setGeometry(QtCore.QRect(240, 30, 165, 30))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.BV_settings_CB.sizePolicy().hasHeightForWidth())
        self.BV_settings_CB.setSizePolicy(sizePolicy)
        self.BV_settings_CB.setMinimumSize(QtCore.QSize(165, 30))
        self.BV_settings_CB.setMaximumSize(QtCore.QSize(165, 16777215))
        self.BV_settings_CB.setObjectName("BV_settings_CB")
        self.BV_settings_CB.addItem("")
        self.BV_settings_CB.addItem("")
        self.BV_settings_CB.addItem("")
        self.BV_settings_CB.addItem("")
        self.gridLayout_4.addWidget(self.splitter, 0, 0, 1, 1)
        self.Main_tabs.addTab(self.Validation, "")
        self.Process = QtWidgets.QWidget()
        self.Process.setObjectName("Process")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.Process)
        self.gridLayout_12.setContentsMargins(5, 10, 5, 0)
        self.gridLayout_12.setVerticalSpacing(9)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.groupBox_11 = QtWidgets.QGroupBox(self.Process)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_11.sizePolicy().hasHeightForWidth())
        self.groupBox_11.setSizePolicy(sizePolicy)
        self.groupBox_11.setObjectName("groupBox_11")
        self.gridLayout_25 = QtWidgets.QGridLayout(self.groupBox_11)
        self.gridLayout_25.setContentsMargins(10, 10, 10, 15)
        self.gridLayout_25.setObjectName("gridLayout_25")
        self.gridLayout_26 = QtWidgets.QGridLayout()
        self.gridLayout_26.setHorizontalSpacing(15)
        self.gridLayout_26.setVerticalSpacing(12)
        self.gridLayout_26.setObjectName("gridLayout_26")
        self.label_21 = QtWidgets.QLabel(self.groupBox_11)
        self.label_21.setObjectName("label_21")
        self.gridLayout_26.addWidget(self.label_21, 0, 0, 1, 1)
        self.label_22 = QtWidgets.QLabel(self.groupBox_11)
        self.label_22.setObjectName("label_22")
        self.gridLayout_26.addWidget(self.label_22, 1, 0, 1, 1)
        self.run_start_PB = QtWidgets.QPushButton(self.groupBox_11)
        self.run_start_PB.setEnabled(True)
        self.run_start_PB.setStyleSheet("")
        self.run_start_PB.setObjectName("run_start_PB")
        self.gridLayout_26.addWidget(self.run_start_PB, 0, 2, 1, 1)
        self.run_stop_PB = QtWidgets.QPushButton(self.groupBox_11)
        self.run_stop_PB.setEnabled(True)
        self.run_stop_PB.setObjectName("run_stop_PB")
        self.gridLayout_26.addWidget(self.run_stop_PB, 0, 3, 1, 1)
        self.run_overalprogress_PBar = QtWidgets.QProgressBar(self.groupBox_11)
        self.run_overalprogress_PBar.setEnabled(False)
        self.run_overalprogress_PBar.setStyleSheet("")
        self.run_overalprogress_PBar.setProperty("value", 0)
        self.run_overalprogress_PBar.setTextVisible(True)
        self.run_overalprogress_PBar.setObjectName("run_overalprogress_PBar")
        self.gridLayout_26.addWidget(self.run_overalprogress_PBar, 2, 1, 1, 3)
        self.run_progress_PBar = QtWidgets.QProgressBar(self.groupBox_11)
        self.run_progress_PBar.setEnabled(False)
        self.run_progress_PBar.setStyleSheet("")
        self.run_progress_PBar.setProperty("value", 0)
        self.run_progress_PBar.setObjectName("run_progress_PBar")
        self.gridLayout_26.addWidget(self.run_progress_PBar, 1, 1, 1, 3)
        self.label_23 = QtWidgets.QLabel(self.groupBox_11)
        self.label_23.setObjectName("label_23")
        self.gridLayout_26.addWidget(self.label_23, 2, 0, 1, 1)
        self.run_progress_TE = QtWidgets.QPlainTextEdit(self.groupBox_11)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.run_progress_TE.sizePolicy().hasHeightForWidth())
        self.run_progress_TE.setSizePolicy(sizePolicy)
        self.run_progress_TE.setMaximumSize(QtCore.QSize(16777215, 80))
        self.run_progress_TE.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.run_progress_TE.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.run_progress_TE.setMidLineWidth(3)
        self.run_progress_TE.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.run_progress_TE.setTabChangesFocus(True)
        self.run_progress_TE.setUndoRedoEnabled(False)
        self.run_progress_TE.setReadOnly(True)
        self.run_progress_TE.setTextInteractionFlags(QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.run_progress_TE.setCenterOnScroll(True)
        self.run_progress_TE.setObjectName("run_progress_TE")
        self.gridLayout_26.addWidget(self.run_progress_TE, 0, 1, 1, 1)
        self.gridLayout_25.addLayout(self.gridLayout_26, 0, 0, 1, 1)
        self.gridLayout_12.addWidget(self.groupBox_11, 1, 0, 1, 1)
        self.groupBox_7 = QtWidgets.QGroupBox(self.Process)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_7.sizePolicy().hasHeightForWidth())
        self.groupBox_7.setSizePolicy(sizePolicy)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_18 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_18.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_18.setHorizontalSpacing(6)
        self.gridLayout_18.setObjectName("gridLayout_18")
        spacerItem = QtWidgets.QSpacerItem(398, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_18.addItem(spacerItem, 2, 0, 1, 1)
        self.process_loadsettings_PB = QtWidgets.QPushButton(self.groupBox_7)
        self.process_loadsettings_PB.setObjectName("process_loadsettings_PB")
        self.gridLayout_18.addWidget(self.process_loadsettings_PB, 2, 1, 1, 1)
        self.toolBox = QtWidgets.QToolBox(self.groupBox_7)
        self.toolBox.setEnabled(True)
        self.toolBox.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.toolBox.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.toolBox.setAutoFillBackground(False)
        self.toolBox.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.toolBox.setFrameShadow(QtWidgets.QFrame.Raised)
        self.toolBox.setObjectName("toolBox")
        self.ExpPage = QtWidgets.QWidget()
        self.ExpPage.setGeometry(QtCore.QRect(0, 0, 838, 302))
        self.ExpPage.setObjectName("ExpPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.ExpPage)
        self.gridLayout_2.setContentsMargins(8, 0, 8, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout_13 = QtWidgets.QGridLayout()
        self.gridLayout_13.setHorizontalSpacing(10)
        self.gridLayout_13.setVerticalSpacing(8)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.label_24 = QtWidgets.QLabel(self.ExpPage)
        self.label_24.setObjectName("label_24")
        self.gridLayout_13.addWidget(self.label_24, 0, 0, 1, 1)
        self.exp_directory_TB = QtWidgets.QToolButton(self.ExpPage)
        self.exp_directory_TB.setObjectName("exp_directory_TB")
        self.gridLayout_13.addWidget(self.exp_directory_TB, 0, 6, 1, 1)
        self.exp_experiments_LE = QtWidgets.QLineEdit(self.ExpPage)
        self.exp_experiments_LE.setText("")
        self.exp_experiments_LE.setObjectName("exp_experiments_LE")
        self.gridLayout_13.addWidget(self.exp_experiments_LE, 1, 1, 1, 6)
        self.label_14 = QtWidgets.QLabel(self.ExpPage)
        self.label_14.setObjectName("label_14")
        self.gridLayout_13.addWidget(self.label_14, 2, 0, 1, 1)
        self.exp_runs_LE = QtWidgets.QLineEdit(self.ExpPage)
        self.exp_runs_LE.setText("")
        self.exp_runs_LE.setObjectName("exp_runs_LE")
        self.gridLayout_13.addWidget(self.exp_runs_LE, 2, 1, 1, 6)
        self.label_25 = QtWidgets.QLabel(self.ExpPage)
        self.label_25.setObjectName("label_25")
        self.gridLayout_13.addWidget(self.label_25, 1, 0, 1, 1)
        self.exp_directory_LE = QtWidgets.QLineEdit(self.ExpPage)
        self.exp_directory_LE.setText("")
        self.exp_directory_LE.setObjectName("exp_directory_LE")
        self.gridLayout_13.addWidget(self.exp_directory_LE, 0, 1, 1, 5)
        self.label = QtWidgets.QLabel(self.ExpPage)
        self.label.setObjectName("label")
        self.gridLayout_13.addWidget(self.label, 3, 0, 1, 1)
        self.exp_patA_LE = QtWidgets.QLineEdit(self.ExpPage)
        self.exp_patA_LE.setObjectName("exp_patA_LE")
        self.gridLayout_13.addWidget(self.exp_patA_LE, 3, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.ExpPage)
        self.label_2.setObjectName("label_2")
        self.gridLayout_13.addWidget(self.label_2, 3, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.ExpPage)
        self.label_3.setObjectName("label_3")
        self.gridLayout_13.addWidget(self.label_3, 3, 4, 1, 1)
        self.exp_patB_LE = QtWidgets.QLineEdit(self.ExpPage)
        self.exp_patB_LE.setObjectName("exp_patB_LE")
        self.gridLayout_13.addWidget(self.exp_patB_LE, 3, 3, 1, 1)
        self.exp_nfiles_LE = QtWidgets.QLineEdit(self.ExpPage)
        self.exp_nfiles_LE.setObjectName("exp_nfiles_LE")
        self.gridLayout_13.addWidget(self.exp_nfiles_LE, 3, 5, 1, 2)
        self.gridLayout_2.addLayout(self.gridLayout_13, 0, 0, 1, 1)
        self.toolBox.addItem(self.ExpPage, "")
        self.PrePage = QtWidgets.QWidget()
        self.PrePage.setGeometry(QtCore.QRect(0, 0, 815, 302))
        self.PrePage.setObjectName("PrePage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.PrePage)
        self.gridLayout_3.setContentsMargins(8, 0, 8, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_7 = QtWidgets.QGridLayout()
        self.gridLayout_7.setHorizontalSpacing(10)
        self.gridLayout_7.setVerticalSpacing(8)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.pre_bg_ncpus_LE = QtWidgets.QLineEdit(self.PrePage)
        self.pre_bg_ncpus_LE.setObjectName("pre_bg_ncpus_LE")
        self.gridLayout_7.addWidget(self.pre_bg_ncpus_LE, 0, 3, 1, 2)
        self.pre_staticmask_CB = QtWidgets.QCheckBox(self.PrePage)
        self.pre_staticmask_CB.setObjectName("pre_staticmask_CB")
        self.gridLayout_7.addWidget(self.pre_staticmask_CB, 1, 0, 1, 1)
        self.pre_sm_path_LE = QtWidgets.QLineEdit(self.PrePage)
        self.pre_sm_path_LE.setObjectName("pre_sm_path_LE")
        self.gridLayout_7.addWidget(self.pre_sm_path_LE, 1, 1, 1, 3)
        self.pre_bg_nfiles_LE = QtWidgets.QLineEdit(self.PrePage)
        self.pre_bg_nfiles_LE.setObjectName("pre_bg_nfiles_LE")
        self.gridLayout_7.addWidget(self.pre_bg_nfiles_LE, 0, 1, 1, 1)
        self.pre_background_CB = QtWidgets.QCheckBox(self.PrePage)
        self.pre_background_CB.setObjectName("pre_background_CB")
        self.gridLayout_7.addWidget(self.pre_background_CB, 0, 0, 1, 1)
        self.pre_bg_chunksize_LE = QtWidgets.QLineEdit(self.PrePage)
        self.pre_bg_chunksize_LE.setObjectName("pre_bg_chunksize_LE")
        self.gridLayout_7.addWidget(self.pre_bg_chunksize_LE, 0, 2, 1, 1)
        self.pre_dynamicmask_CB = QtWidgets.QCheckBox(self.PrePage)
        self.pre_dynamicmask_CB.setObjectName("pre_dynamicmask_CB")
        self.gridLayout_7.addWidget(self.pre_dynamicmask_CB, 2, 0, 1, 3)
        self.pre_sm_path_TB = QtWidgets.QToolButton(self.PrePage)
        self.pre_sm_path_TB.setObjectName("pre_sm_path_TB")
        self.gridLayout_7.addWidget(self.pre_sm_path_TB, 1, 4, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout_7, 0, 0, 1, 1)
        self.toolBox.addItem(self.PrePage, "")
        self.ProPage = QtWidgets.QWidget()
        self.ProPage.setGeometry(QtCore.QRect(0, 0, 838, 302))
        self.ProPage.setObjectName("ProPage")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.ProPage)
        self.gridLayout_8.setContentsMargins(8, 0, 8, 0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.gridLayout_9 = QtWidgets.QGridLayout()
        self.gridLayout_9.setHorizontalSpacing(10)
        self.gridLayout_9.setVerticalSpacing(8)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.pro_scale_LE = QtWidgets.QLineEdit(self.ProPage)
        self.pro_scale_LE.setText("")
        self.pro_scale_LE.setPlaceholderText("")
        self.pro_scale_LE.setObjectName("pro_scale_LE")
        self.gridLayout_9.addWidget(self.pro_scale_LE, 1, 6, 1, 1)
        self.pro_searcharea_LE = QtWidgets.QLineEdit(self.ProPage)
        self.pro_searcharea_LE.setText("")
        self.pro_searcharea_LE.setPlaceholderText("")
        self.pro_searcharea_LE.setObjectName("pro_searcharea_LE")
        self.gridLayout_9.addWidget(self.pro_searcharea_LE, 0, 4, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.ProPage)
        self.label_8.setObjectName("label_8")
        self.gridLayout_9.addWidget(self.label_8, 2, 0, 1, 2)
        self.label_7 = QtWidgets.QLabel(self.ProPage)
        self.label_7.setObjectName("label_7")
        self.gridLayout_9.addWidget(self.label_7, 0, 5, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.ProPage)
        self.label_10.setObjectName("label_10")
        self.gridLayout_9.addWidget(self.label_10, 1, 3, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.ProPage)
        self.label_12.setObjectName("label_12")
        self.gridLayout_9.addWidget(self.label_12, 1, 5, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.ProPage)
        self.label_11.setObjectName("label_11")
        self.gridLayout_9.addWidget(self.label_11, 1, 0, 1, 2)
        self.pro_sig2noise_CB = QtWidgets.QComboBox(self.ProPage)
        self.pro_sig2noise_CB.setObjectName("pro_sig2noise_CB")
        self.pro_sig2noise_CB.addItem("")
        self.pro_sig2noise_CB.addItem("")
        self.pro_sig2noise_CB.addItem("")
        self.gridLayout_9.addWidget(self.pro_sig2noise_CB, 1, 2, 1, 1)
        self.pro_timestep_LE = QtWidgets.QLineEdit(self.ProPage)
        self.pro_timestep_LE.setText("")
        self.pro_timestep_LE.setPlaceholderText("")
        self.pro_timestep_LE.setObjectName("pro_timestep_LE")
        self.gridLayout_9.addWidget(self.pro_timestep_LE, 1, 4, 1, 1)
        self.pro_ncpus_LE = QtWidgets.QLineEdit(self.ProPage)
        self.pro_ncpus_LE.setText("")
        self.pro_ncpus_LE.setPlaceholderText("")
        self.pro_ncpus_LE.setObjectName("pro_ncpus_LE")
        self.gridLayout_9.addWidget(self.pro_ncpus_LE, 2, 2, 1, 1)
        self.pro_overlap_LE = QtWidgets.QLineEdit(self.ProPage)
        self.pro_overlap_LE.setText("")
        self.pro_overlap_LE.setPlaceholderText("")
        self.pro_overlap_LE.setObjectName("pro_overlap_LE")
        self.gridLayout_9.addWidget(self.pro_overlap_LE, 0, 6, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.ProPage)
        self.label_4.setObjectName("label_4")
        self.gridLayout_9.addWidget(self.label_4, 0, 3, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.ProPage)
        self.label_9.setObjectName("label_9")
        self.gridLayout_9.addWidget(self.label_9, 0, 0, 1, 2)
        self.pro_windowsize_LE = QtWidgets.QLineEdit(self.ProPage)
        self.pro_windowsize_LE.setText("")
        self.pro_windowsize_LE.setPlaceholderText("")
        self.pro_windowsize_LE.setObjectName("pro_windowsize_LE")
        self.gridLayout_9.addWidget(self.pro_windowsize_LE, 0, 2, 1, 1)
        self.gridLayout_8.addLayout(self.gridLayout_9, 0, 0, 1, 1)
        self.toolBox.addItem(self.ProPage, "")
        self.PosPage = QtWidgets.QWidget()
        self.PosPage.setGeometry(QtCore.QRect(0, 0, 838, 302))
        self.PosPage.setObjectName("PosPage")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.PosPage)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.groupBox_9 = QtWidgets.QGroupBox(self.PosPage)
        self.groupBox_9.setObjectName("groupBox_9")
        self.gridLayout_21 = QtWidgets.QGridLayout(self.groupBox_9)
        self.gridLayout_21.setContentsMargins(8, 10, 8, 5)
        self.gridLayout_21.setObjectName("gridLayout_21")
        self.gridLayout_22 = QtWidgets.QGridLayout()
        self.gridLayout_22.setObjectName("gridLayout_22")
        self.pos_std_LE = QtWidgets.QLineEdit(self.groupBox_9)
        self.pos_std_LE.setObjectName("pos_std_LE")
        self.gridLayout_22.addWidget(self.pos_std_LE, 1, 1, 1, 1)
        self.pos_s2n_ratio_LE = QtWidgets.QLineEdit(self.groupBox_9)
        self.pos_s2n_ratio_LE.setEnabled(True)
        self.pos_s2n_ratio_LE.setText("")
        self.pos_s2n_ratio_LE.setFrame(True)
        self.pos_s2n_ratio_LE.setObjectName("pos_s2n_ratio_LE")
        self.gridLayout_22.addWidget(self.pos_s2n_ratio_LE, 0, 1, 1, 1)
        self.pos_gv_ulim_LE = QtWidgets.QLineEdit(self.groupBox_9)
        self.pos_gv_ulim_LE.setObjectName("pos_gv_ulim_LE")
        self.gridLayout_22.addWidget(self.pos_gv_ulim_LE, 0, 3, 1, 1)
        self.pos_std_CB = QtWidgets.QCheckBox(self.groupBox_9)
        self.pos_std_CB.setEnabled(True)
        self.pos_std_CB.setObjectName("pos_std_CB")
        self.gridLayout_22.addWidget(self.pos_std_CB, 1, 0, 1, 1)
        self.pos_localvelocity_CB = QtWidgets.QCheckBox(self.groupBox_9)
        self.pos_localvelocity_CB.setEnabled(True)
        self.pos_localvelocity_CB.setObjectName("pos_localvelocity_CB")
        self.gridLayout_22.addWidget(self.pos_localvelocity_CB, 1, 2, 1, 1)
        self.pos_lv_uvdiff_LE = QtWidgets.QLineEdit(self.groupBox_9)
        self.pos_lv_uvdiff_LE.setObjectName("pos_lv_uvdiff_LE")
        self.gridLayout_22.addWidget(self.pos_lv_uvdiff_LE, 1, 3, 1, 1)
        self.pos_sig2noise_CB = QtWidgets.QCheckBox(self.groupBox_9)
        self.pos_sig2noise_CB.setEnabled(True)
        self.pos_sig2noise_CB.setObjectName("pos_sig2noise_CB")
        self.gridLayout_22.addWidget(self.pos_sig2noise_CB, 0, 0, 1, 1)
        self.pos_globalvelocity_CB = QtWidgets.QCheckBox(self.groupBox_9)
        self.pos_globalvelocity_CB.setEnabled(True)
        self.pos_globalvelocity_CB.setObjectName("pos_globalvelocity_CB")
        self.gridLayout_22.addWidget(self.pos_globalvelocity_CB, 0, 2, 1, 1)
        self.pos_lv_kernel_LE = QtWidgets.QLineEdit(self.groupBox_9)
        self.pos_lv_kernel_LE.setObjectName("pos_lv_kernel_LE")
        self.gridLayout_22.addWidget(self.pos_lv_kernel_LE, 1, 4, 1, 1)
        self.pos_gv_vlim_LE = QtWidgets.QLineEdit(self.groupBox_9)
        self.pos_gv_vlim_LE.setObjectName("pos_gv_vlim_LE")
        self.gridLayout_22.addWidget(self.pos_gv_vlim_LE, 0, 4, 1, 1)
        self.gridLayout_21.addLayout(self.gridLayout_22, 0, 0, 1, 1)
        self.gridLayout_10.addWidget(self.groupBox_9, 0, 0, 1, 1)
        self.groupBox_10 = QtWidgets.QGroupBox(self.PosPage)
        self.groupBox_10.setObjectName("groupBox_10")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.groupBox_10)
        self.gridLayout_14.setContentsMargins(9, -1, -1, -1)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.gridLayout_6 = QtWidgets.QGridLayout()
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.pos_badvector_CB = QtWidgets.QCheckBox(self.groupBox_10)
        self.pos_badvector_CB.setEnabled(True)
        self.pos_badvector_CB.setObjectName("pos_badvector_CB")
        self.gridLayout_6.addWidget(self.pos_badvector_CB, 0, 0, 1, 2)
        self.pos_bv_niterations_LE = QtWidgets.QLineEdit(self.groupBox_10)
        self.pos_bv_niterations_LE.setObjectName("pos_bv_niterations_LE")
        self.gridLayout_6.addWidget(self.pos_bv_niterations_LE, 0, 3, 1, 1)
        self.pos_bv_method_CB = QtWidgets.QComboBox(self.groupBox_10)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pos_bv_method_CB.sizePolicy().hasHeightForWidth())
        self.pos_bv_method_CB.setSizePolicy(sizePolicy)
        self.pos_bv_method_CB.setObjectName("pos_bv_method_CB")
        self.pos_bv_method_CB.addItem("")
        self.pos_bv_method_CB.addItem("")
        self.pos_bv_method_CB.addItem("")
        self.gridLayout_6.addWidget(self.pos_bv_method_CB, 0, 2, 1, 1)
        self.pos_bv_kernel_LE = QtWidgets.QLineEdit(self.groupBox_10)
        self.pos_bv_kernel_LE.setObjectName("pos_bv_kernel_LE")
        self.gridLayout_6.addWidget(self.pos_bv_kernel_LE, 0, 4, 1, 1)
        self.pos_smth_factor_LE = QtWidgets.QLineEdit(self.groupBox_10)
        self.pos_smth_factor_LE.setObjectName("pos_smth_factor_LE")
        self.gridLayout_6.addWidget(self.pos_smth_factor_LE, 1, 2, 1, 1)
        self.pos_smoothing_CB = QtWidgets.QCheckBox(self.groupBox_10)
        self.pos_smoothing_CB.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pos_smoothing_CB.sizePolicy().hasHeightForWidth())
        self.pos_smoothing_CB.setSizePolicy(sizePolicy)
        self.pos_smoothing_CB.setObjectName("pos_smoothing_CB")
        self.gridLayout_6.addWidget(self.pos_smoothing_CB, 1, 0, 1, 2)
        self.pos_fm_LE = QtWidgets.QLineEdit(self.groupBox_10)
        self.pos_fm_LE.setObjectName("pos_fm_LE")
        self.gridLayout_6.addWidget(self.pos_fm_LE, 2, 2, 1, 1)
        self.pos_fieldmanip_CB = QtWidgets.QCheckBox(self.groupBox_10)
        self.pos_fieldmanip_CB.setEnabled(True)
        self.pos_fieldmanip_CB.setObjectName("pos_fieldmanip_CB")
        self.gridLayout_6.addWidget(self.pos_fieldmanip_CB, 2, 0, 1, 2)
        self.gridLayout_14.addLayout(self.gridLayout_6, 0, 0, 1, 1)
        self.gridLayout_10.addWidget(self.groupBox_10, 1, 0, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(6)
        self.horizontalLayout_3.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_3.setContentsMargins(5, 10, 25, -1)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_15 = QtWidgets.QLabel(self.PosPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_15.sizePolicy().hasHeightForWidth())
        self.label_15.setSizePolicy(sizePolicy)
        self.label_15.setMinimumSize(QtCore.QSize(150, 0))
        self.label_15.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_3.addWidget(self.label_15)
        self.pos_output_CB = QtWidgets.QComboBox(self.PosPage)
        self.pos_output_CB.setObjectName("pos_output_CB")
        self.pos_output_CB.addItem("")
        self.pos_output_CB.addItem("")
        self.horizontalLayout_3.addWidget(self.pos_output_CB)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.horizontalLayout_3.setStretch(1, 1)
        self.horizontalLayout_3.setStretch(2, 2)
        self.gridLayout_10.addLayout(self.horizontalLayout_3, 2, 0, 1, 1)
        self.toolBox.addItem(self.PosPage, "")
        self.gridLayout_18.addWidget(self.toolBox, 0, 0, 1, 3)
        self.process_savesettings_PB = QtWidgets.QPushButton(self.groupBox_7)
        self.process_savesettings_PB.setObjectName("process_savesettings_PB")
        self.gridLayout_18.addWidget(self.process_savesettings_PB, 2, 2, 1, 1)
        self.gridLayout_12.addWidget(self.groupBox_7, 0, 0, 1, 1)
        self.Main_tabs.addTab(self.Process, "")
        self.FrequencyAnalysis = QtWidgets.QWidget()
        self.FrequencyAnalysis.setObjectName("FrequencyAnalysis")
        self.label_13 = QtWidgets.QLabel(self.FrequencyAnalysis)
        self.label_13.setGeometry(QtCore.QRect(10, 20, 471, 22))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        self.Main_tabs.addTab(self.FrequencyAnalysis, "")
        self.ModalAnalysis = QtWidgets.QWidget()
        self.ModalAnalysis.setObjectName("ModalAnalysis")
        self.label_6 = QtWidgets.QLabel(self.ModalAnalysis)
        self.label_6.setGeometry(QtCore.QRect(10, 20, 471, 22))
        font = QtGui.QFont()
        font.setPointSize(9)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.Main_tabs.addTab(self.ModalAnalysis, "")
        self.gridLayout_11.addWidget(self.Main_tabs, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 876, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.actionLoad_Files = QtWidgets.QAction(MainWindow)
        self.actionLoad_Files.setObjectName("actionLoad_Files")
        self.actionExit = QtWidgets.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionClear_Files = QtWidgets.QAction(MainWindow)
        self.actionClear_Files.setObjectName("actionClear_Files")
        self.actionLoad_Raw_Images = QtWidgets.QAction(MainWindow)
        self.actionLoad_Raw_Images.setObjectName("actionLoad_Raw_Images")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.menuFile.addAction(self.actionLoad_Files)
        self.menuFile.addAction(self.actionLoad_Raw_Images)
        self.menuFile.addAction(self.actionClear_Files)
        self.menuFile.addAction(self.actionExit)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.Main_tabs.setCurrentIndex(1)
        self.toolBox.setCurrentIndex(3)
        self.toolBox.layout().setSpacing(4)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        self.settings_GB.setTitle(QtWidgets.QApplication.translate("MainWindow", "Thresholding Settings", None, -1))
        self.global_velocity_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Global Velocity", None, -1))
        self.global_uVelocity_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Umin, Umax", None, -1))
        self.global_vVelocity_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Vmin, Vmax", None, -1))
        self.local_velocity_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Local Median", None, -1))
        self.local_velocity_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Udiff, Vdiff", None, -1))
        self.local_kernel_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "kernel size", None, -1))
        self.s2n_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Signal 2 Noise", None, -1))
        self.global_std_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Global Std. Deviation", None, -1))
        self.s2n_LE.setWhatsThis(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p><br/></p></body></html>", None, -1))
        self.s2n_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "signal/noise", None, -1))
        self.global_std_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "std multiplier", None, -1))
        self.apply_settings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Apply Settings", None, -1))
        self.load_settings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Load Settings", None, -1))
        self.save_settings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Save Settings", None, -1))
        self.files_TW.setSortingEnabled(True)
        self.files_TW.headerItem().setText(0, QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
        self.files_TW.headerItem().setText(1, QtWidgets.QApplication.translate("MainWindow", "Num. Bad Vector", None, -1))
        self.plot_settings_LE.setText(QtWidgets.QApplication.translate("MainWindow", "b,r,50,2", None, -1))
        self.plot_settings_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "b,r,50,2", None, -1))
        self.BV_settings_CB.setItemText(0, QtWidgets.QApplication.translate("MainWindow", "Show Original BV", None, -1))
        self.BV_settings_CB.setItemText(1, QtWidgets.QApplication.translate("MainWindow", "Show Replaced BV", None, -1))
        self.BV_settings_CB.setItemText(2, QtWidgets.QApplication.translate("MainWindow", "Do Not Show BV", None, -1))
        self.BV_settings_CB.setItemText(3, QtWidgets.QApplication.translate("MainWindow", "Only Show BV", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.Validation), QtWidgets.QApplication.translate("MainWindow", "Validation", None, -1))
        self.groupBox_11.setTitle(QtWidgets.QApplication.translate("MainWindow", "Run", None, -1))
        self.label_21.setText(QtWidgets.QApplication.translate("MainWindow", "Progress", None, -1))
        self.label_22.setText(QtWidgets.QApplication.translate("MainWindow", "Run Progress", None, -1))
        self.run_start_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Start", None, -1))
        self.run_stop_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Stop", None, -1))
        self.label_23.setText(QtWidgets.QApplication.translate("MainWindow", "Overal Progress", None, -1))
        self.run_progress_TE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Details of the run are shown here...", None, -1))
        self.groupBox_7.setTitle(QtWidgets.QApplication.translate("MainWindow", "Run Settings", None, -1))
        self.process_loadsettings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Load Settings", None, -1))
        self.label_24.setText(QtWidgets.QApplication.translate("MainWindow", "Directory", None, -1))
        self.exp_directory_TB.setText(QtWidgets.QApplication.translate("MainWindow", "...", None, -1))
        self.exp_experiments_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Experiments separated by a comma (,)", None, -1))
        self.label_14.setText(QtWidgets.QApplication.translate("MainWindow", "Runs", None, -1))
        self.exp_runs_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Runs separated by a comma (,)", None, -1))
        self.label_25.setText(QtWidgets.QApplication.translate("MainWindow", "Experiments", None, -1))
        self.exp_directory_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Path to the experiment directory", None, -1))
        self.label.setText(QtWidgets.QApplication.translate("MainWindow", "Pattern A", None, -1))
        self.exp_patA_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Pattern for frame_a", None, -1))
        self.label_2.setText(QtWidgets.QApplication.translate("MainWindow", "Pattern B", None, -1))
        self.label_3.setText(QtWidgets.QApplication.translate("MainWindow", "Num. of files", None, -1))
        self.exp_patB_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Pattern for frame_b", None, -1))
        self.exp_nfiles_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Number of files in each run", None, -1))
        self.toolBox.setItemText(self.toolBox.indexOf(self.ExpPage), QtWidgets.QApplication.translate("MainWindow", "Experiment", None, -1))
        self.pre_bg_ncpus_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Number of CPUs", None, -1))
        self.pre_staticmask_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Static Mask", None, -1))
        self.pre_sm_path_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Path to the mask image", None, -1))
        self.pre_bg_nfiles_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Number of files to consider", None, -1))
        self.pre_background_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Remove Background", None, -1))
        self.pre_bg_chunksize_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Chunk size", None, -1))
        self.pre_dynamicmask_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Dynamic Mask (still under development)", None, -1))
        self.pre_sm_path_TB.setText(QtWidgets.QApplication.translate("MainWindow", "...", None, -1))
        self.toolBox.setItemText(self.toolBox.indexOf(self.PrePage), QtWidgets.QApplication.translate("MainWindow", "Pre-Process", None, -1))
        self.label_8.setText(QtWidgets.QApplication.translate("MainWindow", "Numb. CPU Cores", None, -1))
        self.label_7.setText(QtWidgets.QApplication.translate("MainWindow", "Overlap", None, -1))
        self.label_10.setText(QtWidgets.QApplication.translate("MainWindow", "Time Step", None, -1))
        self.label_12.setText(QtWidgets.QApplication.translate("MainWindow", "Scale", None, -1))
        self.label_11.setText(QtWidgets.QApplication.translate("MainWindow", "Signal/Noise Method", None, -1))
        self.pro_sig2noise_CB.setItemText(0, QtWidgets.QApplication.translate("MainWindow", "peak2peak", None, -1))
        self.pro_sig2noise_CB.setItemText(1, QtWidgets.QApplication.translate("MainWindow", "peak2mean", None, -1))
        self.pro_sig2noise_CB.setItemText(2, QtWidgets.QApplication.translate("MainWindow", "None", None, -1))
        self.label_4.setText(QtWidgets.QApplication.translate("MainWindow", "Search Area Size", None, -1))
        self.label_9.setText(QtWidgets.QApplication.translate("MainWindow", "Window Size", None, -1))
        self.toolBox.setItemText(self.toolBox.indexOf(self.ProPage), QtWidgets.QApplication.translate("MainWindow", "Process", None, -1))
        self.groupBox_9.setTitle(QtWidgets.QApplication.translate("MainWindow", "Validation", None, -1))
        self.pos_std_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "std multiplier", None, -1))
        self.pos_s2n_ratio_LE.setWhatsThis(QtWidgets.QApplication.translate("MainWindow", "<html><head/><body><p><br/></p></body></html>", None, -1))
        self.pos_s2n_ratio_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "signal/noise", None, -1))
        self.pos_gv_ulim_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Umin, Umax", None, -1))
        self.pos_std_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Global Std. Deviation", None, -1))
        self.pos_localvelocity_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Local Median", None, -1))
        self.pos_lv_uvdiff_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Udiff, Vdiff", None, -1))
        self.pos_sig2noise_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Signal 2 Noise", None, -1))
        self.pos_globalvelocity_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Global Velocity", None, -1))
        self.pos_lv_kernel_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "kernel size", None, -1))
        self.pos_gv_vlim_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Vmin, Vmax", None, -1))
        self.groupBox_10.setTitle(QtWidgets.QApplication.translate("MainWindow", "Vector Corrections", None, -1))
        self.pos_badvector_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Bad Vectors Replacement", None, -1))
        self.pos_bv_niterations_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Num. of max iterations", None, -1))
        self.pos_bv_method_CB.setItemText(0, QtWidgets.QApplication.translate("MainWindow", "localmean", None, -1))
        self.pos_bv_method_CB.setItemText(1, QtWidgets.QApplication.translate("MainWindow", "disk", None, -1))
        self.pos_bv_method_CB.setItemText(2, QtWidgets.QApplication.translate("MainWindow", "distance", None, -1))
        self.pos_bv_kernel_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "Kernel size", None, -1))
        self.pos_smth_factor_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "smoothing coeff. u, v", None, -1))
        self.pos_smoothing_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Vector Smoothing", None, -1))
        self.pos_fm_LE.setPlaceholderText(QtWidgets.QApplication.translate("MainWindow", "ex. rotCW, flipUD", None, -1))
        self.pos_fieldmanip_CB.setText(QtWidgets.QApplication.translate("MainWindow", "Field Manipulations", None, -1))
        self.label_15.setText(QtWidgets.QApplication.translate("MainWindow", "Output Mode", None, -1))
        self.pos_output_CB.setItemText(0, QtWidgets.QApplication.translate("MainWindow", "simple", None, -1))
        self.pos_output_CB.setItemText(1, QtWidgets.QApplication.translate("MainWindow", "extended", None, -1))
        self.toolBox.setItemText(self.toolBox.indexOf(self.PosPage), QtWidgets.QApplication.translate("MainWindow", "Post-Process", None, -1))
        self.process_savesettings_PB.setText(QtWidgets.QApplication.translate("MainWindow", "Save Settings", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.Process), QtWidgets.QApplication.translate("MainWindow", "Process", None, -1))
        self.label_13.setText(QtWidgets.QApplication.translate("MainWindow", "This part of the program is still under development.", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.FrequencyAnalysis), QtWidgets.QApplication.translate("MainWindow", "Freq. Analysis", None, -1))
        self.label_6.setText(QtWidgets.QApplication.translate("MainWindow", "This part of the program is still under development.", None, -1))
        self.Main_tabs.setTabText(self.Main_tabs.indexOf(self.ModalAnalysis), QtWidgets.QApplication.translate("MainWindow", "Modal Analysis", None, -1))
        self.menuFile.setTitle(QtWidgets.QApplication.translate("MainWindow", "File", None, -1))
        self.menuHelp.setTitle(QtWidgets.QApplication.translate("MainWindow", "Help", None, -1))
        self.actionLoad_Files.setText(QtWidgets.QApplication.translate("MainWindow", "Load Files", None, -1))
        self.actionExit.setText(QtWidgets.QApplication.translate("MainWindow", "Exit", None, -1))
        self.actionClear_Files.setText(QtWidgets.QApplication.translate("MainWindow", "Clear Files", None, -1))
        self.actionLoad_Raw_Images.setText(QtWidgets.QApplication.translate("MainWindow", "Load Raw Images", None, -1))
        self.actionAbout.setText(QtWidgets.QApplication.translate("MainWindow", "About", None, -1))

